@@ -5,10 +5,10 @@ import ssl
 import certifi
 from urllib.parse import urlencode, urlparse
 
-from requests import Request, Session, status_codes
+from requests import status_codes, adapters, PreparedRequest
 
 from Tea.vendored import aiohttp
-from Tea.exceptions import TeaException, RequiredArgumentException
+from Tea.exceptions import TeaException, RequiredArgumentException, RetryError
 from Tea.model import TeaModel
 from Tea.request import TeaRequest
 from Tea.response import TeaResponse
@@ -18,6 +18,8 @@ from typing import Dict, Any, Optional
 
 DEFAULT_CONNECT_TIMEOUT = 5000
 DEFAULT_READ_TIMEOUT = 10000
+DEFAULT_POOL_SIZE = 10
+
 
 logger = logging.getLogger('alibabacloud-tea')
 logger.setLevel(logging.DEBUG)
@@ -26,6 +28,16 @@ logger.addHandler(ch)
 
 
 class TeaCore:
+    http_adapter = adapters.HTTPAdapter(DEFAULT_POOL_SIZE, DEFAULT_POOL_SIZE)
+    https_adapter = adapters.HTTPAdapter(DEFAULT_POOL_SIZE, DEFAULT_POOL_SIZE)
+
+    @staticmethod
+    def get_adapter(prefix):
+        if prefix.upper() == 'HTTP':
+            return TeaCore.http_adapter
+        else:
+            return TeaCore.https_adapter
+
     @staticmethod
     def _prepare_http_debug(request, symbol):
         base = ''
@@ -127,18 +139,21 @@ class TeaCore:
                     body += content
             else:
                 body = request.body
-            async with s.request(request.method, url,
-                                 data=body,
-                                 headers=request.headers,
-                                 ssl=verify,
-                                 proxy=proxy,
-                                 timeout=timeout) as response:
-                tea_resp = TeaResponse()
-                tea_resp.body = await response.read()
-                tea_resp.headers = {k.lower(): v for k, v in response.headers.items()}
-                tea_resp.status_code = response.status
-                tea_resp.status_message = response.reason
-                tea_resp.response = response
+            try:
+                async with s.request(request.method, url,
+                                     data=body,
+                                     headers=request.headers,
+                                     ssl=verify,
+                                     proxy=proxy,
+                                     timeout=timeout) as response:
+                    tea_resp = TeaResponse()
+                    tea_resp.body = await response.read()
+                    tea_resp.headers = {k.lower(): v for k, v in response.headers.items()}
+                    tea_resp.status_code = response.status
+                    tea_resp.status_message = response.reason
+                    tea_resp.response = response
+            except IOError as e:
+                raise RetryError(str(e))
         return tea_resp
 
     @staticmethod
@@ -159,46 +174,54 @@ class TeaCore:
         read_timeout = read_timeout if read_timeout else DEFAULT_READ_TIMEOUT
 
         timeout = (int(connect_timeout) / 1000, int(read_timeout) / 1000)
-        with Session() as s:
-            req = Request(method=request.method, url=url,
-                          data=request.body, headers=request.headers)
-            prepped = s.prepare_request(req)
 
-            proxies = {}
-            http_proxy = runtime_option.get('httpProxy')
-            https_proxy = runtime_option.get('httpsProxy')
-            no_proxy = runtime_option.get('noProxy')
+        p = PreparedRequest()
+        p.prepare(
+            method=request.method.upper(),
+            url=url,
+            data=request.body,
+            headers=request.headers,
+        )
 
-            if not http_proxy:
-                http_proxy = os.environ.get('HTTP_PROXY') or os.environ.get('http_proxy')
-            if not https_proxy:
-                https_proxy = os.environ.get('HTTPS_PROXY') or os.environ.get('https_proxy')
+        proxies = {}
+        http_proxy = runtime_option.get('httpProxy')
+        https_proxy = runtime_option.get('httpsProxy')
+        no_proxy = runtime_option.get('noProxy')
 
-            if http_proxy:
-                proxies['http'] = http_proxy
-            if https_proxy:
-                proxies['https'] = https_proxy
-            if no_proxy:
-                proxies['no_proxy'] = no_proxy
+        if not http_proxy:
+            http_proxy = os.environ.get('HTTP_PROXY') or os.environ.get('http_proxy')
+        if not https_proxy:
+            https_proxy = os.environ.get('HTTPS_PROXY') or os.environ.get('https_proxy')
 
-            resp = s.send(
-                prepped,
+        if http_proxy:
+            proxies['http'] = http_proxy
+        if https_proxy:
+            proxies['https'] = https_proxy
+        if no_proxy:
+            proxies['no_proxy'] = no_proxy
+
+        adapter = TeaCore.get_adapter(request.protocol)
+        try:
+            resp = adapter.send(
+                p,
                 proxies=proxies,
                 timeout=timeout,
                 verify=verify,
             )
+        except IOError as e:
+            raise RetryError(str(e))
 
-            debug = runtime_option.get('debug') or os.getenv('DEBUG')
-            if debug and debug.lower() == 'sdk':
-                TeaCore._do_http_debug(req, resp)
+        debug = runtime_option.get('debug') or os.getenv('DEBUG')
+        if debug and debug.lower() == 'sdk':
+            TeaCore._do_http_debug(p, resp)
 
-            response = TeaResponse()
-            response.status_message = resp.reason
-            response.status_code = resp.status_code
-            response.headers = {k.lower(): v for k, v in resp.headers.items()}
-            response.body = resp.content
-            response.response = resp
-            return response
+        response = TeaResponse()
+        response.status_message = resp.reason
+        response.status_code = resp.status_code
+        response.headers = {k.lower(): v for k, v in resp.headers.items()}
+        response.body = resp.content
+        response.response = resp
+        return response
 
     @staticmethod
     def get_response_body(resp) -> str:
@@ -206,7 +229,8 @@ class TeaCore:
 
     @staticmethod
     def allow_retry(dic, retry_times, now=None) -> bool:
-        if dic is None or not dic.__contains__("maxAttempts"):
+        if dic is None or not dic.__contains__("maxAttempts") or \
+                dic.get('retryable') is not True and retry_times >= 1:
             return False
         else:
             retry = 0 if dic.get("maxAttempts") is None else int(
@@ -236,7 +260,7 @@ class TeaCore:
 
     @staticmethod
     def is_retryable(ex) -> bool:
-        return isinstance(ex, TeaException)
+        return isinstance(ex, RetryError)
 
     @staticmethod
     def bytes_readable(body):
@@ -265,6 +289,10 @@ class TeaCore:
             dic: Dict[str, Any]
     ) -> TeaModel:
         if isinstance(model, TeaModel):
-            return model.from_map(dic)
+            try:
+                return model.from_map(dic)
+            except Exception:
+                model._map = dic
+                return model
         else:
             return model
